/*
 Andrei Speridião
 6451022
 
 Projeto Final - Fundamentos da Computação II
 
 -----------------
 
 Baseado em:
 
 - Script de tracking de JMyron: http://webcamxtra.sourceforge.net/
 - Exemplos nativos do Processing: constraint, easing.
 - Libraries nativas do Processing: Minim(sound)
 
 
 <<<<<<<<<<<<<<<<OBSERVAÇÕES>>>>>>>>>>>>>>>>>>>
 1 - Para acessar as configurações de câmera, clique com o botão direito sobre a tela.
 
 Utilizar os seguintes settings de camera:
 equilibrio automatico de luz desligado
 rotação horizontal
 exposição: < 50
 
 2 - Para esconder/mostrar o video, as caixas de colisão e de tracking, clique com o botão esquerdo sobre a tela.
 
 */

//importa a biblioteca de tracking
import JMyron.*;
JMyron cam;

//importa a biblioteca de audio
import ddf.minim.*;
Minim minim;
AudioSample kick;


//declara as variaveis do pong
float x = 50; //posição horizontal da bolinha
float y = 50; //posição vertical da bolinha
float incX = 10; //incremento horizontal da bolinha
float incY = 5;  //incremento vertical da bolinha
int largBola = 20; // largura da bolinha
float raqueteX; //posição horizontal da raquete (atualmente fixo)
float raqueteY; //posição vertical da raquete
int largRaquete = 10; //largura da raquete
int altRaquete = 100; //altura da raquete
float mediaX; //media horizontal calculada a partir dos pontos extremos captados pelo tracking
float mediaY; //media vertical calculada a partir dos pontos extremos captados pelo tracking
float easing = 0.2; //quantidade damping de movimento (quanto maior o numero, menos damping)
float mx; //variavel utilizada para apicar o damping horizontalmente (desativado)
float my; //variavel utilizada para apicar o damping verticalmente
int colisorRaqX; //armazena o ponto de colisão horizontal ao longo da raquete
int colisorRaqY; //armazena o ponto de colisão vertical ao longo da raquete
int vcam = 1; //cria a variável que controla a exibição da camera e caixas suporte.

//inicia o setup
void setup(){
  size(640,480);

  //audio
  minim = new Minim(this);
  kick = minim.loadSample("BD.mp3", 2048);

  //camera tracking
  cam = new JMyron();//declara a captura de camera
  cam.start(width,height);//inicia a captura de video a 640x480
  cam.trackColor(255,255,255,255);//R, G, B, e o campo de tolerancia
  cam.minDensity(300); //minimo numero de pixels em um lote para gerar um bracket de tracking independente
  background(0);  

}


//rotina draw
void draw(){

  if (vcam < 0)
  {
    cameraShow(); 
  }

  cam.update();//atualiza a captura de video
  int[][] b = cam.globBoxes();//captura os pontos de tracking
  noFill();

  //desenha as caixas volumetricas
  stroke(255,0,0);
  for(int i=0;i<b.length;i++){
    int p1 = b[i][0]; //X superior esquerdo
    //      println(p1);

    int p2 = b[i][1]; //Y superior esquerdo
    //      println(p2);

    int p3 = b[i][2]; // largura da caixa
    //      println(p3); 

    int p4 = b[i][3]; // altura da caixa
    //      println(p4);  

    int p5 = p1+p3;  //calcula o ponto X inferior direito
    int p6 = p2+p4;  //calcula o ponto Y inferior direito

    //      rect( p1 , p2 , p1+p3 , p2+p4 );  //desenha a caixa volumetrica com os novos pontos
    //      rect( p1 , p2 , 20 , 20 );  //desenha uma caixa com o centro correspondente ao ponto esquerdo superior
    //      rect( p5 , p6 , 20 , 20 );  //desenha uma caixa com o centro correspondente ao ponto direito inferior

    mediaX = (p1+p5)/2;  //calcula o centro horizontal entre as extremidades trackeadas
    mediaY = (p2+p6)/2;  //calcula o centro vertical entre as extremidades trackeadas



  }

  //inicia a engine do pong

  //  a movimentação da raquete em x foi desativada, mas convem manter este código
  //  if(abs(mediaX - mx) > 0.1) {
  //    mx = mx + (mediaX - mx) * easing * 2;
  //  } 

  // damping de movimentação da raquete (deixa mais natural o movimento captado pela web cam)
  if(abs(mediaY - my) > 0.1) {
    my = my + (mediaY - my) * easing;
  }


  raqueteX = width-100; ///fixa o movimento horizontal
  raqueteY = constrain(my, altRaquete/2, height - altRaquete/2);  //restringe o movimento vertical à tela

  rectMode (CENTER);


  //caso nao esteja no modo 2 cria efeito de motion blur
  if (vcam >0)
  {
    noStroke();
    fill(0,100); 
    rect(width/2,height/2,width,height);
  }
  
  //desenha a raquete
  noStroke();
  fill(255);
  rect (raqueteX, raqueteY, largRaquete, altRaquete);

  // movimenta a bolinha 
  x = x + incX;
  y = y + incY;


  // interação da bolinha com o perimetro da tela  
  if (x <= largBola/2 || x >= width - largBola/2)
  {
    incX = - incX;
    kick.trigger();    
  }
  if (y <= largBola/2 || y >= height - largBola/2)
  {
    incY = - incY;
    kick.trigger(); 
  }


  // interação da bolinha com a raquete
  float minY = raqueteY - altRaquete/2 - largBola/2;
  float maxY = raqueteY + altRaquete/2 + largBola/2;
  float minX = raqueteX - largRaquete/2 - largBola/2;
  float maxX = raqueteX + largRaquete/2 + largBola/2;

  //somente se satisfeitas as condições de colisao com a raquete:
  //em x ambas as laterais, e em y, cima e baixo, é invertida o sentido 
  if (x >= minX && x <= maxX && y >= minY && y <= maxY){
    incX = - incX;
    kick.trigger();   
  } 


  //desenha a bolinha
  rect ( x, y, 10, 10);

  // infelizmente este modo de execução de som não suportou pan(efeito de profundidade).
  // tentei implementar o pan de acordo com a posição da bolinha no espaço X
  //  if ( kick.hasControl(Controller.PAN) )
  //  {
  //    float val = map(x, 0, width, -1, 1);
  //    // if a pan control is not available, this will do nothing
  //    kick.setPan(val);
  //  }  


}  //fim da engine do pong




// função que controla a exibição da camera e das caixas de suporte
void cameraShow()
{
  int[] img = cam.image(); //get the normal image of the camera
  loadPixels();
  for(int i=0;i<width*height;i++) //loop through all the pixels
    pixels[i] = img[i]; //draw each pixel to the screen
  updatePixels();

  noFill();
  stroke (255,0,0,100);  
  rectMode (CENTER);
  rect( mediaX , mediaY , 20 , 20 );  //desenha uma caixa volumetrica baseada no centro horizontal e vertical

  rectMode (CORNER);
  rect (0,0,width,raqueteY + altRaquete/2);
  rect (0,0,width,raqueteY - altRaquete/2);
  rect (0,0,raqueteX + largRaquete/2, height);
  rect (0,0,raqueteX - largRaquete/2, height);
}




void mousePressed()
{  
  if (mouseButton == RIGHT)
  {
    cam.settings(); //clique direito na tela para acessar as configurações da camera
  }
  if (mouseButton == LEFT)
  {
    vcam = -vcam; //inverte a variavel vcam, para toggle de exibição da camera e caixas suporte
  }  
}


public void stop()
{
  cam.stop();//stop the object
  super.stop();  
}



